﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.IO;

namespace S01VIN1Sample
{
    public partial class Form1 : Form
    {
        private StreamWriter SaveFile; //保存ファイルストリーム
        private Boolean SamplingStopFlg; //中止指示フラグ Trueで中止
        private short SamplingStatus; //動作状態 0:停止 1:トリガ待ち 2:データ取得中
        private short[] tmpSampleDataBuf; //データ取り込み用
        private int SampledLen; //取り込み済みデータ数

        public Form1()
        {
            InitializeComponent();
        }

        // 画面表示制御
        // state 0:クローズ状態 1:オープン状態 2:サンプリング中
        private void EnableDisableCont(int state)
        {
            switch (state)
            {
                case 0:// クローズ
                    Timer_Set_Btn.Enabled = false;
                    Input_Set_Btn.Enabled = false;
                    Sampling_Start_Btn.Enabled = false;
                    Sampling_Stop_Btn.Enabled = false;
                    Soft_Trigger_Btn.Enabled = false;
                    ID_SEL.Enabled = true;
                    OPEN_CLOSE_Btn.Enabled = true;
                    break;
                case 1:// オープン　サンプリング停止時
                    Timer_Set_Btn.Enabled = true;
                    Input_Set_Btn.Enabled = true;
                    Sampling_Start_Btn.Enabled = true;
                    Sampling_Stop_Btn.Enabled = false;
                    Soft_Trigger_Btn.Enabled = false;
                    ID_SEL.Enabled = false;
                    OPEN_CLOSE_Btn.Enabled = true;
                    break;
                case 2:// オープン　サンプリング動作時
                    Timer_Set_Btn.Enabled = false;
                    Input_Set_Btn.Enabled = false;
                    Sampling_Start_Btn.Enabled = false;
                    Sampling_Stop_Btn.Enabled = true;
                    Soft_Trigger_Btn.Enabled = true;
                    ID_SEL.Enabled = false;
                    OPEN_CLOSE_Btn.Enabled = false;
                    break;
            }
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            SamplingStatus = 0; //サンプリング動作状態　停止
            ID_SEL.SelectedIndex = 0; //IDの初期値を0とする
            Clock_M.Text = "480"; //タイマー1/480
            Clock_N.SelectedIndex = 0; //タイマー指数部 0
            GAIN.SelectedIndex = 0; //GAIN ×1
            TriggerSel.SelectedIndex = 0; //トリガ選択 ソフトトリガ
            TriggerLevel.Text = "0"; //トリガレベル 0
            PreTriggerLen.Text = "0"; //プレトリガ長 0
            BufferLen.Text = "1000"; //取り込み長1000
            EnableDisableCont(0); //画面イネーブル制御
            tmpSampleDataBuf = new short[100000];//作業用バッファ取得
        }

        private void Form1_FormClosed(object sender, FormClosedEventArgs e)
        {
            if (OPEN_CLOSE_Btn.Text == "CLOSE")//オープンされている時
            {
                if (Sampling_Stop_Btn.Enabled)//サンプリング動作中
                {
                    SamplingStop();//停止
                }
                DeviceClose(); //デバイスを閉じる
            }
        }

        private void Timer_Set_Btn_Click(object sender, EventArgs e)
        {
            short ret;
            byte N;
            int M;

            N = (byte)Clock_N.SelectedIndex; //乗数Nの取得
            if (int.TryParse(Clock_M.Text, out M))//整数Mのチェック
            {
                ret = TUSBSVI1.Tusbs01vin1_Timer_Set((short)ID_SEL.SelectedIndex, N, M); //タイマー設定
                //結果の表示
                if (ret == 0)
                    StatusMessage.Text = "<Timer_Set> Complete.";
                else
                    MessageBox.Show(TUSBSVI1.GetErrMessage(ret));
            }
            else
            {
                MessageBox.Show("Please input an integer into 'M'.");
            }
        }

        private void Input_Set_Btn_Click(object sender, EventArgs e)
        {
            short ret;

            ret = TUSBSVI1.Tusbs01vin1_Input_Set((short)ID_SEL.SelectedIndex, (byte)GAIN.SelectedIndex); //設定

            //結果の表示
            if (ret == 0)
                StatusMessage.Text = "<Input_Set> Complete.";
            else
                MessageBox.Show(TUSBSVI1.GetErrMessage(ret));
        }

        private void Sampling_Start_Btn_Click(object sender, EventArgs e)
        {
            short ret;
            short Level;
            int PreLen;
            int TotalLen;

            if (short.TryParse(TriggerLevel.Text, out Level) == false) // トリガレベル入力
            {
                MessageBox.Show("Please input an integer into 'Trigger Level'.");
                return;
            }

            if (int.TryParse(PreTriggerLen.Text, out PreLen) == false) //プレトリガ長入力
            {
                MessageBox.Show("Please input an integer into 'Pre Trigger'.");
                return;
            }

            if (int.TryParse(BufferLen.Text, out TotalLen) == false) // バッファ長入力
            {
                MessageBox.Show("Please input an integer into 'Buffer Len'.");
                return;
            }

            SaveFileDialog1.OverwritePrompt = true;
            SaveFileDialog1.Filter = "データファイル(*.csv)|*.csv";

            if (SaveFileDialog1.ShowDialog() == DialogResult.OK)//保存ファイル名等を取得
            {
                if (System.IO.File.Exists(SaveFileDialog1.FileName))//既存ファイルチェック
                {
                    System.IO.File.Delete(SaveFileDialog1.FileName);
                }
                SaveFile = new StreamWriter(File.OpenWrite(SaveFileDialog1.FileName));//ファイルを開く

                ret = TUSBSVI1.Tusbs01vin1_Sampling_Start((short)ID_SEL.SelectedIndex, (byte)TriggerSel.SelectedIndex, Level, PreLen, TotalLen); //スタート

                if (ret == 0)//正常に開始できた場合
                {
                    StatusMessage.Text = "Waiting for trigger.";
                    EnableDisableCont(2); //画面表示制御
                    SamplingStopFlg = false; //中止指示フラグ初期化
                    SamplingStatus = 1; //トリガ待ちステータス
                    Timer1.Interval = 100; //100mSタイマー
                    SampledLen = 0;
                    Timer1.Enabled = true; //タイマー開始
                }
                else
                {
                    SaveFile.Close();
                    MessageBox.Show(TUSBSVI1.GetErrMessage(ret));//エラー発生
                }
            }
        }

        private void Sampling_Stop_Btn_Click(object sender, EventArgs e)
        {
            SamplingStopFlg = true; //中止指示フラグ
        }

        private void Soft_Trigger_Btn_Click(object sender, EventArgs e)
        {
            short ret;

            ret = TUSBSVI1.Tusbs01vin1_Sampling_Trigger((short)ID_SEL.SelectedIndex);//トリガ発生

            // 結果の表示
            if (ret != 0)
            {
                MessageBox.Show(TUSBSVI1.GetErrMessage(ret));
            }
        }

        //デバイスのオープンに関わる処理
        private void DeviceOpen()
        {
            short ret;
            ret = TUSBSVI1.Tusbs01vin1_Device_Open((short)ID_SEL.SelectedIndex);//開く

            //結果の表示
            if (ret == 0)
            {
                EnableDisableCont(1); //正常にオープンできた
                OPEN_CLOSE_Btn.Text = "CLOSE";
            }
            else //オープンできなかった
            {
                MessageBox.Show(TUSBSVI1.GetErrMessage(ret));//エラーメッセージ表示
            }
        }

        //デバイスのクローズに関わる処理
        private void DeviceClose()
        {
            TUSBSVI1.Tusbs01vin1_Device_Close((short)ID_SEL.SelectedIndex);//開く
            EnableDisableCont(0); // 画面表示制御
            OPEN_CLOSE_Btn.Text = "OPEN";
        }

        private void OPEN_CLOSE_Btn_Click(object sender, EventArgs e)
        {
            if (OPEN_CLOSE_Btn.Text == "OPEN")//OPENならば
                DeviceOpen();
            else
                DeviceClose();
        }
        // 取り込み停止時の処理
        private void SamplingStop()
        {
            int cnt;
            short ret;
            int leng;

            ret = TUSBSVI1.Tusbs01vin1_Sampling_Stop((short)ID_SEL.SelectedIndex);//取り込み停止

            // 結果の表示
            if (ret != 0)
                MessageBox.Show(TUSBSVI1.GetErrMessage(ret));
            else
                StatusMessage.Text = "<Sampling> Stop";

            //最終データの保存
            do
            {
                leng = 100000;
                ret = TUSBSVI1.Tusbs01vin1_Memory_Read((short)ID_SEL.SelectedIndex, tmpSampleDataBuf, ref leng);//読み出し
                if (ret != 0)
                    MessageBox.Show(TUSBSVI1.GetErrMessage(ret));
                else
                {
                    for (cnt = 0; cnt < leng; cnt++)
                    {
                        SaveFile.WriteLine(tmpSampleDataBuf[cnt]);//保存
                    }
                }
            } while ((leng != 0) && (ret == 0));//長さ0以外でかつエラーでない場合
            SaveFile.Close();//ファイルを閉じる
            EnableDisableCont(1); //画面表示制御
        }

        private void Timer1_Tick(object sender, EventArgs e)
        {
            short ret;
            byte status;
            byte ovf;
            int leng;
            int cnt;

            switch (SamplingStatus)
            {
                case 0://停止状態
                    Timer1.Enabled = false;
                    break;
                case 1://トリガ待ち状態
                    if (SamplingStopFlg)//停止要求
                    {
                        Timer1.Enabled = false;//タイマー停止
                        SamplingStop();//取り込み停止
                    }
                    else
                    {
                        ret = TUSBSVI1.Tusbs01vin1_Status_Read((short)ID_SEL.SelectedIndex, out status, out ovf, out leng);//状態確認

                        //結果の表示
                        if (ret != 0)
                        {
                            MessageBox.Show(TUSBSVI1.GetErrMessage(ret));//エラーメッセージ
                            SamplingStop();//取り込み停止
                            SamplingStatus = 0;//サンプリング状態を0にする
                        }
                        else
                        {
                            switch (status)
                            {
                                case 0://なんからの理由で停止状態になった
                                    Timer1.Enabled = false;//タイマー停止
                                    SamplingStop();//取り込み停止
                                    SamplingStatus = 0;//サンプリング状態を0にする
                                    break;
                                case 2://サンプリング中になった
                                    StatusMessage.Text = "<Sampling>";
                                    SamplingStatus = 2;//サンプリング状態を2にする
                                    break;
                            }
                        }
                    }
                    break;
                case 2://取り込み中
                    if (SamplingStopFlg)//停止要求
                    {
                        Timer1.Enabled = false;//タイマー停止
                        SamplingStop();//取り込み停止
                    }
                    else
                    {
                        ret = TUSBSVI1.Tusbs01vin1_Status_Read((short)ID_SEL.SelectedIndex, out status, out ovf, out leng); //停止状態

                        //結果の表示
                        if (ret != 0)
                        {
                            MessageBox.Show(TUSBSVI1.GetErrMessage(ret));//エラーメッセージ
                            SamplingStop(); //取り込み停止
                            SamplingStatus = 0; //サンプリング状態を0にする
                        }
                        else
                        {
                            switch (status)
                            {
                                case 0://なんらかの理由で停止状態になった
                                    Timer1.Enabled = false;//タイマー停止
                                    SamplingStop();//取り込み停止
                                    SamplingStatus = 0;//サンプリング状態を0にする
                                    break;
                                case 2://サンプリング中
                                    leng = 100000;//一時取得バッファの長さ
                                    ret = TUSBSVI1.Tusbs01vin1_Memory_Read((short)ID_SEL.SelectedIndex, tmpSampleDataBuf, ref leng);//データ取り込み
                                    for (cnt = 0; cnt < leng; cnt++)
                                    {
                                        SaveFile.WriteLine(tmpSampleDataBuf[cnt]);//ファイルへの書き込み
                                    }
                                    SampledLen = SampledLen + leng;
                                    StatusMessage.Text = SampledLen.ToString();
                                    break;
                            }
                        }
                    }
                    break;
            }
        }
    }
}
